@@ -39,13 +39,24 @@ PROJECT_ROOT = TESTS_ROOT.parent
 RESOURCE_ROOT = PROJECT_ROOT / "resources"
 
 
-def test_float_object_exception():
+class ChildDummy(DictionaryObject):
+    @property
+    def indirect_ref(self):
+        return self
+
+
+def test_float_object_exception(caplog):
     assert FloatObject("abc") == 0
+    assert caplog.text != ""
 
 
-def test_number_object_exception():
-    with pytest.raises(OverflowError):
-        NumberObject(1.5 * 2**10000)
+def test_number_object_exception(caplog):
+    assert NumberObject("0,0") == 0
+    assert caplog.text != ""
+
+
+def test_number_object_no_exception():
+    NumberObject(2**100000000)
 
 
 def test_create_string_object_exception():
@@ -165,16 +176,65 @@ def test_readStringFromStream_excape_digit2():
     assert read_string_from_stream(stream) == "hello \x01\x02\x03\x04"
 
 
-def test_NameObject():
+def test_NameObject(caplog):
     stream = BytesIO(b"x")
     with pytest.raises(PdfReadError) as exc:
         NameObject.read_from_stream(stream, None)
     assert exc.value.args[0] == "name read error"
+    assert (
+        NameObject.read_from_stream(
+            BytesIO(b"/A;Name_With-Various***Characters?"), None
+        )
+        == "/A;Name_With-Various***Characters?"
+    )
+    assert (
+        NameObject.read_from_stream(BytesIO(b"/paired#28#29parentheses"), None)
+        == "/paired()parentheses"
+    )
+    assert NameObject.read_from_stream(BytesIO(b"/A#42"), None) == "/AB"
+
+    assert (
+        NameObject.read_from_stream(
+            BytesIO(b"/#f1j#d4#aa#0c#ce#87#b4#b3#b0#23J#86#fe#2a#b2jYJ#94"),
+            ReaderDummy(),
+        )
+        == "/ñjÔª\x0cÎ\x87´³°#J\x86þ*²jYJ\x94"
+    )
+
+    assert (NameObject.read_from_stream(BytesIO(b"/#JA#231f"), None)) == "/#JA#1f"
+
+    assert (
+        NameObject.read_from_stream(
+            BytesIO(b"/#e4#bd#a0#e5#a5#bd#e4#b8#96#e7#95#8c"), None
+        )
+    ) == "/你好世界"
+
+    # test write
+    b = BytesIO()
+    NameObject("/hello").write_to_stream(b, None)
+    assert bytes(b.getbuffer()) == b"/hello"
+
+    caplog.clear()
+    b = BytesIO()
+    NameObject("hello").write_to_stream(b, None)
+    assert bytes(b.getbuffer()) == b"hello"
+    assert "Incorrect first char" in caplog.text
+
+    caplog.clear()
+    b = BytesIO()
+    NameObject("/DIJMAC+Arial Black#1").write_to_stream(b, None)
+    assert bytes(b.getbuffer()) == b"/DIJMAC+Arial#20Black#231"
+    assert caplog.text == ""
+
+    b = BytesIO()
+    NameObject("/你好世界").write_to_stream(b, None)
+    assert bytes(b.getbuffer()) == b"/#E4#BD#A0#E5#A5#BD#E4#B8#96#E7#95#8C"
+    assert caplog.text == ""
 
 
 def test_destination_fit_r():
     d = Destination(
-        NameObject("title"),
+        TextStringObject("title"),
         NullObject(),
         NameObject(TF.FIT_R),
         FloatObject(0),
@@ -215,7 +275,7 @@ def test_outline_item_write_to_stream():
     )
     oi.write_to_stream(stream, None)
     stream.seek(0, 0)
-    assert stream.read() == b"<<\n/Title title\n/Dest [ null /FitV 0 ]\n>>"
+    assert stream.read() == b"<<\n/Title (title)\n/Dest [ null /FitV 0 ]\n>>"
 
 
 def test_encode_pdfdocencoding_keyerror():
@@ -373,6 +433,10 @@ def test_RectangleObject():
     ro.lower_left = (5, 6)
     assert ro.lower_left == (5, 6)
 
+    ro.bottom -= 2
+    ro.left -= 2
+    assert ro.lower_left == (3, 4)
+
     ro.lower_right = (7, 8)
     assert ro.lower_right == (7, 8)
 
@@ -381,6 +445,9 @@ def test_RectangleObject():
 
     ro.upper_right = (13, 17)
     assert ro.upper_right == (13, 17)
+    ro.top += 1
+    ro.right += 1
+    assert ro.upper_right == (14, 18)
 
 
 def test_TextStringObject_exc():
@@ -397,24 +464,22 @@ def test_TextStringObject_autodetect_utf16():
 
 
 def test_remove_child_not_in_tree():
+
     tree = TreeObject()
     with pytest.raises(ValueError) as exc:
-        tree.remove_child(NameObject("foo"))
+        tree.remove_child(ChildDummy())
     assert exc.value.args[0] == "Removed child does not appear to be a tree item"
 
 
 def test_remove_child_not_in_that_tree():
-    class ChildDummy:
-        def __init__(self, parent):
-            self.parent = parent
-
-        def get_object(self):
-            tree = DictionaryObject()
-            tree[NameObject("/Parent")] = self.parent
-            return tree
 
     tree = TreeObject()
-    child = ChildDummy(TreeObject())
+    tree.indirect_ref = NullObject()
+    # child = ChildDummy(TreeObject())
+    child = TreeObject()
+    with pytest.raises(ValueError) as exc:
+        child.remove_from_tree()
+    assert exc.value.args[0] == "Removed child does not appear to be a tree item"
     tree.add_child(child, ReaderDummy())
     with pytest.raises(ValueError) as exc:
         tree.remove_child(child)
@@ -422,20 +487,19 @@ def test_remove_child_not_in_that_tree():
 
 
 def test_remove_child_not_found_in_tree():
-    class ChildDummy:
-        def __init__(self, parent):
-            self.parent = parent
-
-        def get_object(self):
-            tree = DictionaryObject()
-            tree[NameObject("/Parent")] = self.parent
-            return tree
+    class ChildDummy(DictionaryObject):
+        @property
+        def indirect_ref(self):
+            return self
 
     tree = TreeObject()
-    child = ChildDummy(tree)
+    tree.indirect_ref = NullObject()
+    child = ChildDummy(TreeObject())
     tree.add_child(child, ReaderDummy())
+    child2 = ChildDummy(TreeObject())
+    child2[NameObject("/Parent")] = tree
     with pytest.raises(ValueError) as exc:
-        tree.remove_child(child)
+        tree.remove_child(child2)
     assert exc.value.args[0] == "Removal couldn't find item in tree"
 
 
@@ -465,7 +529,7 @@ def test_remove_child_found_in_tree():
     assert len([el for el in tree.children()]) == 2
 
     # Remove last child
-    tree.remove_child(child2)
+    tree.remove_child(child2_ref)
     assert tree[NameObject("/Count")] == 1
     assert len([el for el in tree.children()]) == 1
 
@@ -498,7 +562,8 @@ def test_remove_child_found_in_tree():
     assert len([el for el in tree.children()]) == 3
 
     # Remove middle child
-    tree.remove_child(child4)
+    # tree.remove_child(child4)
+    child4.remove_from_tree()
     assert tree[NameObject("/Count")] == 2
     assert len([el for el in tree.children()]) == 2
 
@@ -511,6 +576,7 @@ def test_remove_child_in_tree():
     tree = TreeObject()
     reader = PdfReader(pdf)
     writer = PdfWriter()
+    writer._add_object(tree)
     writer.add_page(reader.pages[0])
     writer.add_outline_item("foo", pagenum=0)
     obj = writer._objects[-1]
@@ -520,6 +586,7 @@ def test_remove_child_in_tree():
     tree.empty_tree()
 
 
+@pytest.mark.external
 def test_dict_read_from_stream(caplog):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/984/984877.pdf"
     name = "tika-984877.pdf"
@@ -533,6 +600,7 @@ def test_dict_read_from_stream(caplog):
     )
 
 
+@pytest.mark.external
 def test_parse_content_stream_peek_percentage():
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/985/985770.pdf"
     name = "tika-985770.pdf"
@@ -542,6 +610,7 @@ def test_parse_content_stream_peek_percentage():
         page.extract_text()
 
 
+@pytest.mark.external
 def test_read_inline_image_no_has_q():
     # pdf/df7e1add3156af17a372bc165e47a244.pdf
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/998/998719.pdf"
@@ -552,6 +621,7 @@ def test_read_inline_image_no_has_q():
         page.extract_text()
 
 
+@pytest.mark.external
 def test_read_inline_image_loc_neg_1():
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/935/935066.pdf"
     name = "tika-935066.pdf"
@@ -561,6 +631,8 @@ def test_read_inline_image_loc_neg_1():
         page.extract_text()
 
 
+@pytest.mark.slow
+@pytest.mark.external
 def test_text_string_write_to_stream():
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/924/924562.pdf"
     name = "tika-924562.pdf"
@@ -570,6 +642,7 @@ def test_text_string_write_to_stream():
         page.compress_content_streams()
 
 
+@pytest.mark.external
 def test_name_object_read_from_stream_unicode_error():  # L588
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/974/974966.pdf"
     name = "tika-974966.pdf"
@@ -579,19 +652,27 @@ def test_name_object_read_from_stream_unicode_error():  # L588
         page.extract_text()
 
 
-def test_bool_repr():
+@pytest.mark.external
+def test_bool_repr(tmp_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/932/932449.pdf"
     name = "tika-932449.pdf"
 
     reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
-    with open("tmp-fields-report.txt", "w") as fp:
+    write_path = tmp_path / "tmp-fields-report.txt"
+    with open(write_path, "w") as fp:
         fields = reader.get_fields(fileobj=fp)
     assert fields
-
-    # cleanup
-    os.remove("tmp-fields-report.txt")
+    assert list(fields.keys()) == ["USGPOSignature"]
+    with open(write_path) as fp:
+        data = fp.read()
+    assert data.startswith(
+        "Field Name: USGPOSignature\nField Type: Signature\nField Flags: 1\n"
+        "Value: {'/Type': '/Sig', '/Filter': '/Adobe.PPKLite', "
+        "'/SubFilter':"
+    )
 
 
+@pytest.mark.external
 @patch("PyPDF2._reader.logger_warning")
 def test_issue_997(mock_logger_warning):
     url = "https://github.com/py-pdf/PyPDF2/files/8908874/Exhibit_A-2_930_Enterprise_Zone_Tax_Credits_final.pdf"
@@ -685,7 +766,34 @@ def test_annotation_builder_line():
     writer.add_annotation(0, line_annotation)
 
     # Assert: You need to inspect the file manually
-    target = "annotated-pdf.pd"
+    target = "annotated-pdf.pdf"
+    with open(target, "wb") as fp:
+        writer.write(fp)
+
+    os.remove(target)  # comment this out for manual inspection
+
+
+def test_annotation_builder_square():
+    # Arrange
+    pdf_path = RESOURCE_ROOT / "crazyones.pdf"
+    reader = PdfReader(pdf_path)
+    page = reader.pages[0]
+    writer = PdfWriter()
+    writer.add_page(page)
+
+    # Act
+    square_annotation = AnnotationBuilder.rectangle(
+        rect=(50, 550, 200, 650), interiour_color="ff0000"
+    )
+    writer.add_annotation(0, square_annotation)
+
+    square_annotation = AnnotationBuilder.rectangle(
+        rect=(40, 400, 150, 450),
+    )
+    writer.add_annotation(0, square_annotation)
+
+    # Assert: You need to inspect the file manually
+    target = "annotated-pdf-square.pdf"
     with open(target, "wb") as fp:
         writer.write(fp)
 
@@ -784,7 +892,7 @@ def test_name_object_invalid_decode():
     # strict:
     with pytest.raises(PdfReadError) as exc:
         NameObject.read_from_stream(stream, ReaderDummy(strict=True))
-    assert exc.value.args[0] == "Illegal character in Name Object"
+    assert "Illegal character in Name Object" in exc.value.args[0]
 
     # non-strict:
     stream.seek(0)
@@ -802,3 +910,38 @@ def test_create_string_object_force():
     assert create_string_object(b"Hello World", []) == "Hello World"
     assert create_string_object(b"Hello World", {72: "A"}) == "Aello World"
     assert create_string_object(b"Hello World", "utf8") == "Hello World"
+
+
+@pytest.mark.parametrize(
+    ("value", "expected"),
+    [
+        ("0.000000", "0"),
+        ("0.0", "0"),
+        ("1.0", "1"),
+        ("0.123000", "0.123"),
+        ("0.000123000", "0.000123"),
+        ("0.0", "0"),
+        ("0", "0"),
+        ("1", "1"),
+        ("1.0", "1"),
+        ("1.01", "1.01"),
+        ("1.010", "1.01"),
+        ("0000.0000", "0"),
+        ("0.10101010", "0.1010101"),
+        ("50000000000", "50000000000"),
+        ("99900000000000000123", "99900000000000000123"),
+        ("99900000000000000123.456000", "99900000000000000123.456"),
+        ("0.00000000000000000000123", "0.00000000000000000000123"),
+        ("0.00000000000000000000123000", "0.00000000000000000000123"),
+        (
+            "50032481330523882508234.00000000000000000000123000",
+            "50032481330523882508234.00000000000000000000123",
+        ),
+        (
+            "928457298572093487502198745102973402987412908743.75249875981374981237498213740000",
+            "928457298572093487502198745102973402987412908743.7524987598137498123749821374",
+        ),
+    ],
+)
+def test_float_object_decimal_to_string(value, expected):
+    assert repr(FloatObject(value)) == expected
